@@ -54,7 +54,11 @@ CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 /* Everything using inb/outb, etc needs "compiler.h" */
 #include "compiler.h"
 
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 6
 #include "xf86Resources.h"
+/* Needed by Resources Access Control (RAC) */
+#include "xf86RAC.h"
+#endif
 
 /* Drivers for PCI hardware need this */
 #include "xf86PciInfo.h"
@@ -62,9 +66,6 @@ CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 /* Drivers that need to access the PCI config space directly need this */
 #include "xf86Pci.h"
 
-/* This is used for module versioning */
-#include "xf86Version.h"
-
 /* All drivers using the vgahw module need this */
 #include "vgaHW.h"
 
@@ -74,9 +75,6 @@ CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 /* All drivers implementing backing store need this */
 #include "mibstore.h"
 
-/* All drivers using the mi banking wrapper need this */
-#include "mibank.h"
-
 /* All drivers using the mi colormap manipulation need this */
 #include "micmap.h"
 
@@ -84,9 +82,6 @@ CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 #include "fb.h"
 
-/* Needed by Resources Access Control (RAC) */
-#include "xf86RAC.h"
-
 /* int10 */
 #include "xf86int10.h"
 #include "vbe.h"
@@ -111,10 +106,8 @@ CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include <X11/Xproto.h>
 #include "scrnintstr.h"
 #include "servermd.h"
-#ifdef XFreeXDGA
-#define _XF86DGA_SERVER_
-#include <X11/extensions/xf86dgastr.h>
-#endif
+
+#include <unistd.h>
 
 /* Mandatory functions */
 static const OptionInfoRec *	NEOAvailableOptions(int chipid, int busid);
@@ -325,6 +318,7 @@ static PciChipsets NEOPCIchipsets[] = {
     { -1,	     -1,	     RES_UNDEFINED}
 };
 
+#ifdef HAVE_ISA
 static IsaChipsets NEOISAchipsets[] = {
     { NM2070,               RES_EXCLUSIVE_VGA },
     { NM2090,               RES_EXCLUSIVE_VGA },
@@ -334,10 +328,10 @@ static IsaChipsets NEOISAchipsets[] = {
     { NM2200,               RES_EXCLUSIVE_VGA },
     { -1,			RES_UNDEFINED }
 };
+#endif
 
 /* The options supported by the Neomagic Driver */
 typedef enum {
-    OPTION_NOLINEAR_MODE,
     OPTION_NOACCEL,
     OPTION_SW_CURSOR,
     OPTION_NO_MMIO,
@@ -385,7 +379,6 @@ static const OptionInfoRec NEO_2070_Options[] = {
 };
 
 static const OptionInfoRec NEOOptions[] = {
-    { OPTION_NOLINEAR_MODE,"NoLinear",  OPTV_BOOLEAN,	{0}, FALSE },
     { OPTION_NOACCEL,	"NoAccel",	OPTV_BOOLEAN,	{0}, FALSE },
     { OPTION_SW_CURSOR,	"SWcursor",	OPTV_BOOLEAN,	{0}, FALSE },
     { OPTION_NO_MMIO,	"noMMIO",	OPTV_BOOLEAN,	{0}, FALSE },
@@ -413,79 +406,6 @@ static const OptionInfoRec NEOOptions[] = {
     { -1,                  NULL,           OPTV_NONE,	{0}, FALSE }
 };
 
-/*
- * List of symbols from other modules that this module references.  This
- * list is used to tell the loader that it is OK for symbols here to be
- * unresolved providing that it hasn't been told that they haven't been
- * told that they are essential via a call to xf86LoaderReqSymbols() or
- * xf86LoaderReqSymLists().  The purpose is this is to avoid warnings about
- * unresolved symbols that are not required.
- */
-
-static const char *vgahwSymbols[] = {
-    "vgaHWFreeHWRec",
-    "vgaHWGetHWRec",
-    "vgaHWGetIOBase",
-    "vgaHWGetIndex",
-    "vgaHWInit",
-    "vgaHWLock",
-    "vgaHWMapMem",
-    "vgaHWProtect",
-    "vgaHWRestore",
-    "vgaHWSave",
-    "vgaHWSaveScreenWeak",
-    "vgaHWSetStdFuncs",
-    "vgaHWUnlock",
-    "vgaHWddc1SetSpeedWeak",
-    NULL
-};
-
-static const char *fbSymbols[] = {
-    "fbPictureInit",
-    "fbScreenInit",
-    NULL
-};
-
-static const char *xaaSymbols[] = {
-    "XAACreateInfoRec",
-    "XAADestroyInfoRec",
-    "XAAInit",
-    NULL
-};
-
-static const char *ramdacSymbols[] = {
-    "xf86CreateCursorInfoRec",
-    "xf86DestroyCursorInfoRec",
-    "xf86InitCursor",
-    NULL
-};
-
-static const char *shadowSymbols[] = {
-    "shadowInit",
-    NULL
-};
-
-static const char *ddcSymbols[] = {
-    "xf86DoEDID_DDC1",
-    "xf86DoEDID_DDC2",
-    "xf86PrintEDID",
-    "xf86SetDDCproperties",
-    NULL
-};
-
-static const char *vbeSymbols[] = {
-    "VBEInit",
-    "vbeDoEDID",
-    "vbeFree",
-    NULL
-};
-
-static const char *i2cSymbols[] = {
-    "xf86CreateI2CBusRec",
-    "xf86I2CBusInit",
-    NULL
-};
-
 #ifdef XFree86LOADER
 
 static MODULESETUPPROTO(neoSetup);
@@ -519,18 +439,6 @@ neoSetup(pointer module, pointer opts, int *errmaj, int *errmin)
 	setupDone = TRUE;
         xf86AddDriver(&NEOMAGIC, module, 0);
 
-	/*
-	 * Modules that this driver always requires can be loaded here
-	 * by calling LoadSubModule().
-	 */
-
-	/*
-	 * Tell the loader about symbols from other modules that this module
-	 * might refer to.
-	 */
-	LoaderRefSymLists(vgahwSymbols, fbSymbols, xaaSymbols,
-			  ramdacSymbols, shadowSymbols,
-			  ddcSymbols, vbeSymbols, i2cSymbols, NULL);
 	/*
 	 * The return value must be non-NULL on success even though there
 	 * is no TearDownProc.
@@ -646,7 +554,8 @@ NEOProbe(DriverPtr drv, int flags)
 	    xfree(usedChips);
 	}
     }
-    
+
+#ifdef HAVE_ISA 
     /* Isa Bus */
 
     numUsed = xf86MatchIsaInstances(NEO_NAME,NEOChipsets,NEOISAchipsets,
@@ -677,11 +586,13 @@ NEOProbe(DriverPtr drv, int flags)
       }
       xfree(usedChips);
     }
+#endif
 
     xfree(devSections);
     return foundScreen;
 }
 
+#ifdef HAVE_ISA
 static int
 neoFindIsaDevice(GDevPtr dev)
 {
@@ -708,7 +619,7 @@ neoFindIsaDevice(GDevPtr dev)
 	return -1;
     }
 }
-
+#endif
 
 /* Mandatory */
 Bool
@@ -742,8 +653,6 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
     if (!xf86LoadSubModule(pScrn, "vgahw"))
 	return FALSE;
 
-    xf86LoaderReqSymLists(vgahwSymbols, NULL);    
-
     /*
      * Allocate a vgaHWRec.
      */
@@ -772,7 +681,9 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
     /* This is the general case */
     for (i = 0; i<pScrn->numEntities; i++) {
 	nPtr->pEnt = xf86GetEntityInfo(pScrn->entityList[i]);
+#ifndef XSERVER_LIBPCIACCESS
 	if (nPtr->pEnt->resources) return FALSE;
+#endif
 	nPtr->NeoChipset = nPtr->pEnt->chipset;
 	pScrn->chipset = (char *)xf86TokenToString(NEOChipsets,
 						   nPtr->pEnt->chipset);
@@ -963,7 +874,6 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
     pScrn->monitor = pScrn->confScreen->monitor;
 
     if (xf86LoadSubModule(pScrn, "ddc")) {
-        xf86LoaderReqSymLists(ddcSymbols, NULL);
 #if 1 /* for DDC1 testing */
 	if (!neoDoDDCVBE(pScrn))
 	  if (!neoDoDDC2(pScrn))
@@ -1041,7 +951,6 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 
     xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, nPtr->Options);
 
-    xf86GetOptValBool(nPtr->Options, OPTION_NOLINEAR_MODE,&nPtr->noLinear);
     xf86GetOptValBool(nPtr->Options, OPTION_SW_CURSOR,&nPtr->swCursor);
     xf86GetOptValBool(nPtr->Options, OPTION_NO_MMIO,&nPtr->noMMIO);
     xf86GetOptValBool(nPtr->Options, OPTION_INTERN_DISP,&nPtr->internDisp);
@@ -1167,10 +1076,6 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
     if (nPtr->lcdCenter)
 	xf86DrvMsg(pScrn->scrnIndex,X_CONFIG,
 		   "Video modes are centered on the display\n");
-    if (nPtr->noLinear)
-	xf86DrvMsg(pScrn->scrnIndex,X_CONFIG, "using nonlinear mode\n");
-    else
-	xf86DrvMsg(pScrn->scrnIndex,X_DEFAULT, "using linear mode\n");
     if (nPtr->swCursor)
 	xf86DrvMsg(pScrn->scrnIndex,X_CONFIG, "using sofware cursor\n");
     if (nPtr->noMMIO)
@@ -1184,17 +1089,9 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 	xf86DrvMsg(pScrn->scrnIndex,X_CONFIG,
 		   "Show chache for debugging\n");
     if (nPtr->shadowFB) {
-	if (nPtr->noLinear) {
-    	    xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
-		       "Option \"ShadowFB\" ignored. Not supported without"
-		       " linear addressing\n");
-	    nPtr->shadowFB = FALSE;
-	    nPtr->rotate = 0;
-	} else {
-	    nPtr->noAccel = TRUE;
-	    xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, 
-		    "Using \"Shadow Framebuffer\" - acceleration disabled\n");
-	}
+        nPtr->noAccel = TRUE;
+        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, 
+                "Using \"Shadow Framebuffer\" - acceleration disabled\n");
     }
 
     nPtr->NeoFbMapSize = linearSize * 1024;
@@ -1257,11 +1154,15 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 		           nPtr->NeoMMIOAddr2);
 	    }
 	}
+#ifndef XSERVER_LIBPCIACCESS
 	/* XXX What about VGA resources in OPERATING mode? */
 	if (xf86RegisterResources(nPtr->pEnt->index, NULL, ResExclusive))
 	    RETURN;
+#endif
 	    
-    } else if (nPtr->pEnt->location.type == BUS_ISA) {
+    } 
+#ifndef XSERVER_LIBPCIACCESS
+    else if (nPtr->pEnt->location.type == BUS_ISA) {
 	unsigned int addr;
 	resRange linearRes[] = { {ResExcMemBlock|ResBios|ResBus,0,0},_END };
 	
@@ -1280,12 +1181,15 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 		       "MMIO base address is set at 0x%lX.\n",
 		       nPtr->NeoMMIOAddr);
 	}
+
 	linearRes[0].rBegin = nPtr->NeoLinearAddr;
 	linearRes[1].rEnd = nPtr->NeoLinearAddr + nPtr->NeoFbMapSize - 1;
 	if (xf86RegisterResources(nPtr->pEnt->index,linearRes,ResNone)) {
-	    nPtr->noLinear = TRUE; /* XXX */
+	    RETURN;
 	}
-    } else
+    }
+#endif
+    else
 	RETURN;
 
     if (nPtr->pEnt->device->videoRam != 0) {
@@ -1409,25 +1313,18 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 	RETURN;
     }
 
-    xf86LoaderReqSymLists(fbSymbols, NULL);
-
-    if (!nPtr->noLinear) {
-	if (!xf86LoadSubModule(pScrn, "xaa")) 
-	    RETURN;
-	xf86LoaderReqSymLists(xaaSymbols, NULL);
-    }
+    if (!xf86LoadSubModule(pScrn, "xaa")) 
+        RETURN;
 
     if (nPtr->shadowFB) {
 	if (!xf86LoadSubModule(pScrn, "shadow")) {
 	    RETURN;
 	}
-	xf86LoaderReqSymLists(shadowSymbols, NULL);
     }
     
     if (!nPtr->swCursor) {
 	if (!xf86LoadSubModule(pScrn, "ramdac"))
 	    RETURN;
-	xf86LoaderReqSymLists(ramdacSymbols, NULL);
     }
     return TRUE;
 }
@@ -1517,7 +1414,9 @@ NEOScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     int ret;
     VisualPtr visual;
     int allocatebase, freespace, currentaddr;
+#ifndef XSERVER_LIBPCIACCESS
     unsigned int racflag = RAC_FB;
+#endif
     unsigned char *FBStart;
     int height, width, displayWidth;
     
@@ -1555,22 +1454,11 @@ NEOScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     miClearVisualTypes();
     
     /* Setup the visuals we support. */
-#if 0
     if (!miSetVisualTypes(pScrn->depth,
       		      miGetDefaultVisualMask(pScrn->depth),
 		      pScrn->rgbBits, pScrn->defaultVisual))
          return FALSE;
-#else
-    if (!miSetVisualTypes(pScrn->depth,
-      		      miGetDefaultVisualMask(pScrn->depth),
-		      pScrn->rgbBits, pScrn->defaultVisual))
-         return FALSE;
-    if (pScrn->depth > 8) {
-	if (!miSetVisualTypes(8, miGetDefaultVisualMask(8), 6,
-			      pScrn->defaultVisual))
-	    return FALSE;
-    }
-#endif
+
     if (!miSetPixmapDepths ()) return FALSE;
 
     /*
@@ -1606,8 +1494,7 @@ NEOScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
         /* Fixup RGB ordering */
         visual = pScreen->visuals + pScreen->numVisuals;
         while (--visual >= pScreen->visuals) {
-	    if ((visual->class | DynamicClass) == DirectColor
-		&& visual->nplanes > 8) {
+	    if ((visual->class | DynamicClass) == DirectColor) {
 		visual->offsetRed = pScrn->offset.red;
 		visual->offsetGreen = pScrn->offset.green;
 		visual->offsetBlue = pScrn->offset.blue;
@@ -1631,134 +1518,105 @@ NEOScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     nAcl->UseHWCursor = FALSE;
     nAcl->CursorAddress = -1;
     
-    if (nPtr->noLinear) {
-	miBankInfoPtr pBankInfo;
+    nAcl->cacheStart = -1;
+    nAcl->cacheEnd = -1;
+    xf86DrvMsg(pScrn->scrnIndex,X_INFO,
+               "Using linear framebuffer at: 0x%08lX\n",
+               nPtr->NeoLinearAddr);
+    /* Setup pointers to free space in video ram */
+    allocatebase = (pScrn->videoRam << 10);
+    freespace = allocatebase - pScrn->displayWidth *
+        pScrn->virtualY * (pScrn->bitsPerPixel >> 3);
+    currentaddr = allocatebase;
+    xf86DrvMsg(scrnIndex, X_PROBED,
+               "%d bytes off-screen memory available\n", freespace);
 
-	/* Setup the vga banking variables */
-	pBankInfo = (miBankInfoPtr)xnfcalloc(sizeof(miBankInfoRec),1);
-	if (pBankInfo == NULL)
-	    return FALSE;
-	
-	pBankInfo->pBankA = hwp->Base;
-	pBankInfo->pBankB = (unsigned char *)hwp->Base;
-	pBankInfo->BankSize = 0x10000;
-	pBankInfo->nBankDepth = pScrn->depth;
-	
-	pBankInfo->SetSourceBank = (miBankProcPtr)NEOSetRead;
-	pBankInfo->SetDestinationBank =
-	    (miBankProcPtr)NEOSetWrite;
-	pBankInfo->SetSourceAndDestinationBanks =
-	    (miBankProcPtr)NEOSetReadWrite;
-	if (!miInitializeBanking(pScreen, pScrn->virtualX, pScrn->virtualY,
-				 pScrn->displayWidth, pBankInfo)) {
-	    xfree(pBankInfo);
-	    pBankInfo = NULL;
-	    return FALSE;
-	}
-	xf86DrvMsg(pScrn->scrnIndex,X_INFO, "Using nonlinear mode\n");
-	xf86DrvMsg(pScrn->scrnIndex,X_INFO, "Using software cursor in "
-		   "nonlinear mode\n");
+    if (nPtr->swCursor || !nPtr->NeoMMIOBase) {
+        xf86DrvMsg(scrnIndex, X_CONFIG,
+                   "Using Software Cursor.\n");
+    } else if (nPtr->NeoCursorMem <= freespace) {
+        currentaddr -= nPtr->NeoCursorMem;
+        freespace  -= nPtr->NeoCursorMem;
+        /* alignment */
+        freespace  -= currentaddr & 0x3FF;
+        currentaddr &= 0xfffffc00;
+        nAcl->CursorAddress = currentaddr;
+        xf86DrvMsg(scrnIndex, X_INFO,
+                   "Using H/W Cursor.\n"); 
     } else {
-	nAcl->cacheStart = -1;
-	nAcl->cacheEnd = -1;
-	xf86DrvMsg(pScrn->scrnIndex,X_INFO,
-		   "Using linear framebuffer at: 0x%08lX\n",
-		   nPtr->NeoLinearAddr);
-	/* Setup pointers to free space in video ram */
-	allocatebase = (pScrn->videoRam << 10);
-	freespace = allocatebase - pScrn->displayWidth *
-	    pScrn->virtualY * (pScrn->bitsPerPixel >> 3);
-	currentaddr = allocatebase;
-	xf86DrvMsg(scrnIndex, X_PROBED,
-		   "%d bytes off-screen memory available\n", freespace);
-
-	if (nPtr->swCursor || !nPtr->NeoMMIOBase) {
-	    xf86DrvMsg(scrnIndex, X_CONFIG,
-		       "Using Software Cursor.\n");
-	} else if (nPtr->NeoCursorMem <= freespace) {
-	    currentaddr -= nPtr->NeoCursorMem;
-	    freespace  -= nPtr->NeoCursorMem;
-	    /* alignment */
-	    freespace  -= currentaddr & 0x3FF;
-	    currentaddr &= 0xfffffc00;
-	    nAcl->CursorAddress = currentaddr;
-	    xf86DrvMsg(scrnIndex, X_INFO,
-		       "Using H/W Cursor.\n"); 
-	} else {
-	    xf86DrvMsg(scrnIndex, X_ERROR,
-		       "Too little space for H/W cursor.\n");
-	}
-	
-	if (!nPtr->noAccel && !nPtr->NeoMMIOBase)
-	  xf86DrvMsg(pScrn->scrnIndex,X_INFO,
-		     "Acceleration disabled when not using MMIO\n");
-
-	if (nPtr->overlay > 0){
-	    if (nPtr->overlay > freespace){
-		xf86DrvMsg(pScrn->scrnIndex,X_INFO,
-			   "Can not reserve %d bytes for overlay. "
-			   "Resize to %d bytes.\n",
-			   nPtr->overlay, freespace);
-		nPtr->overlay = freespace;
-	    }
-	    currentaddr -= nPtr->overlay;
-	    freespace -= nPtr->overlay;
-	    nPtr->overlay_offset = currentaddr;
-	    xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Overlay at 0x%x\n",
-		       nPtr->overlay_offset);
-	}
-
-	nAcl->cacheStart = currentaddr - freespace;
-	nAcl->cacheEnd = currentaddr;
-	freespace = 0;
-	if (nAcl->cacheStart < nAcl->cacheEnd) {
-	    BoxRec AvailFBArea;
-	    int lines = nAcl->cacheEnd /
-		(pScrn->displayWidth * (pScrn->bitsPerPixel >> 3));
-	    if (!nPtr->noAccel && nPtr->NeoMMIOBase && lines > 1024) 
-		lines = 1024;
-	    AvailFBArea.x1 = 0;
-	    AvailFBArea.y1 = 0;
-	    AvailFBArea.x2 = pScrn->displayWidth;
-	    AvailFBArea.y2 = lines;
-	    xf86InitFBManager(pScreen, &AvailFBArea); 
-	    
-	    xf86DrvMsg(pScrn->scrnIndex, X_INFO, 
-		       "Using %i scanlines of offscreen memory \n",
-		       lines - pScrn->virtualY);
-	}
-
-	/* Setup the acceleration primitives */
-	if (!nPtr->noAccel && nPtr->NeoMMIOBase) {
-	    Bool ret = FALSE;
-	    if (nAcl->cacheStart >= nAcl->cacheEnd) {
-		xf86DrvMsg(scrnIndex, X_ERROR,
-			   "Too little space for pixmap cache.\n");
-	    } 	    
-	    switch(nPtr->NeoChipset) {
-	    case NM2070 :
-		ret = Neo2070AccelInit(pScreen);
-		break;
-	    case NM2090 :
-	    case NM2093 :
-		ret = Neo2090AccelInit(pScreen);
-		break;
-	    case NM2097 :
-	    case NM2160 :
-		ret = Neo2097AccelInit(pScreen);
-		break;
-	    case NM2200 :
-	    case NM2230 :
-	    case NM2360 :
-	    case NM2380 :
-	        ret = Neo2200AccelInit(pScreen);
-		break;
-	    }
-	    xf86DrvMsg(pScrn->scrnIndex,X_INFO,
-		       "Acceleration %s Initialized\n",ret ? "" : "not");
-	} 
-
+        xf86DrvMsg(scrnIndex, X_ERROR,
+                   "Too little space for H/W cursor.\n");
     }
+    
+    if (!nPtr->noAccel && !nPtr->NeoMMIOBase)
+      xf86DrvMsg(pScrn->scrnIndex,X_INFO,
+                 "Acceleration disabled when not using MMIO\n");
+
+    if (nPtr->overlay > 0){
+        if (nPtr->overlay > freespace){
+            xf86DrvMsg(pScrn->scrnIndex,X_INFO,
+                       "Can not reserve %d bytes for overlay. "
+                       "Resize to %d bytes.\n",
+                       nPtr->overlay, freespace);
+            nPtr->overlay = freespace;
+        }
+        currentaddr -= nPtr->overlay;
+        freespace -= nPtr->overlay;
+        nPtr->overlay_offset = currentaddr;
+        xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Overlay at 0x%x\n",
+                   nPtr->overlay_offset);
+    }
+
+    nAcl->cacheStart = currentaddr - freespace;
+    nAcl->cacheEnd = currentaddr;
+    freespace = 0;
+    if (nAcl->cacheStart < nAcl->cacheEnd) {
+        BoxRec AvailFBArea;
+        int lines = nAcl->cacheEnd /
+            (pScrn->displayWidth * (pScrn->bitsPerPixel >> 3));
+        if (!nPtr->noAccel && nPtr->NeoMMIOBase && lines > 1024) 
+            lines = 1024;
+        AvailFBArea.x1 = 0;
+        AvailFBArea.y1 = 0;
+        AvailFBArea.x2 = pScrn->displayWidth;
+        AvailFBArea.y2 = lines;
+        xf86InitFBManager(pScreen, &AvailFBArea); 
+        
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO, 
+                   "Using %i scanlines of offscreen memory \n",
+                   lines - pScrn->virtualY);
+    }
+
+    /* Setup the acceleration primitives */
+    if (!nPtr->noAccel && nPtr->NeoMMIOBase) {
+        Bool ret = FALSE;
+        if (nAcl->cacheStart >= nAcl->cacheEnd) {
+            xf86DrvMsg(scrnIndex, X_ERROR,
+                       "Too little space for pixmap cache.\n");
+        } 	    
+        switch(nPtr->NeoChipset) {
+        case NM2070 :
+            ret = Neo2070AccelInit(pScreen);
+            break;
+        case NM2090 :
+        case NM2093 :
+            ret = Neo2090AccelInit(pScreen);
+            break;
+        case NM2097 :
+        case NM2160 :
+            ret = Neo2097AccelInit(pScreen);
+            break;
+        case NM2200 :
+        case NM2230 :
+        case NM2360 :
+        case NM2380 :
+            ret = Neo2200AccelInit(pScreen);
+            break;
+        }
+        xf86DrvMsg(pScrn->scrnIndex,X_INFO,
+                   "Acceleration %s Initialized\n",ret ? "" : "not");
+    } 
+
     miInitializeBackingStore(pScreen);
     xf86SetBackingStore(pScreen);
     xf86SetSilkenMouse(pScreen);
@@ -1809,11 +1667,13 @@ NEOScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
                          CMAP_PALETTED_TRUECOLOR | CMAP_RELOAD_ON_MODE_SWITCH))
 	return FALSE;
 
+#ifndef XSERVER_LIBPCIACCESS
     racflag |= RAC_COLORMAP;
     if (nPtr->NeoHWCursorInitialized)
         racflag |= RAC_CURSOR;
 
     pScrn->racIoFlags = pScrn->racMemFlags = racflag;
+#endif
 
     NEOInitVideo(pScreen);
 
@@ -1824,10 +1684,8 @@ NEOScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	xf86DPMSInit(pScreen, (DPMSSetProcPtr)NeoDisplayPowerManagementSet,
 		     0);
 
-    if (!nPtr->noLinear) {
-	pScrn->memPhysBase = (unsigned long)nPtr->NeoLinearAddr;
-	pScrn->fbOffset = 0;
-    }
+    pScrn->memPhysBase = (unsigned long)nPtr->NeoLinearAddr;
+    pScrn->fbOffset = 0;
     
     /* Wrap the current CloseScreen function */
     nPtr->CloseScreen = pScreen->CloseScreen;
@@ -2054,85 +1912,80 @@ neoMapMem(ScrnInfoPtr pScrn)
     NEOPtr nPtr = NEOPTR(pScrn);
     vgaHWPtr hwp = VGAHWPTR(pScrn);
 
-    if (!nPtr->noLinear) {
-	if (!nPtr->noMMIO) {
-	    if (nPtr->pEnt->location.type == BUS_PCI){
+    if (!nPtr->noMMIO) {
+        if (nPtr->pEnt->location.type == BUS_PCI){
 
 #ifndef XSERVER_LIBPCIACCESS
-		nPtr->NeoMMIOBase =
-		    xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO,
-				  nPtr->PciTag, nPtr->NeoMMIOAddr,
-				  0x200000L);
-		if (nPtr->NeoMMIOAddr2 != 0){
-		    nPtr->NeoMMIOBase2 =
-		        xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO,
-				      nPtr->PciTag, nPtr->NeoMMIOAddr2,
-				      0x100000L);
-		}
+            nPtr->NeoMMIOBase =
+                xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO,
+                              nPtr->PciTag, nPtr->NeoMMIOAddr,
+                              0x200000L);
+            if (nPtr->NeoMMIOAddr2 != 0){
+                nPtr->NeoMMIOBase2 =
+                    xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO,
+                                  nPtr->PciTag, nPtr->NeoMMIOAddr2,
+                                  0x100000L);
+            }
 
 #else
-		void** result = (void**)&nPtr->NeoMMIOBase;
-		int err = pci_device_map_range(nPtr->PciInfo,
-					       nPtr->NeoMMIOAddr,
-					       0x200000L,
-					       PCI_DEV_MAP_FLAG_WRITABLE,
-					       result);
-		if (err)
-		    return FALSE;
-		
-		if (nPtr->NeoMMIOAddr2 != 0){
-		    result = (void**)&nPtr->NeoMMIOBase2;
-		    err = pci_device_map_range(nPtr->PciInfo,
-						   nPtr->NeoMMIOAddr2,
-						   0x100000L,
-						   PCI_DEV_MAP_FLAG_WRITABLE,
-						   result);
+            void** result = (void**)&nPtr->NeoMMIOBase;
+            int err = pci_device_map_range(nPtr->PciInfo,
+                                           nPtr->NeoMMIOAddr,
+                                           0x200000L,
+                                           PCI_DEV_MAP_FLAG_WRITABLE,
+                                           result);
+            if (err)
+                return FALSE;
+            
+            if (nPtr->NeoMMIOAddr2 != 0){
+                result = (void**)&nPtr->NeoMMIOBase2;
+                err = pci_device_map_range(nPtr->PciInfo,
+                                               nPtr->NeoMMIOAddr2,
+                                               0x100000L,
+                                               PCI_DEV_MAP_FLAG_WRITABLE,
+                                               result);
 
-		    if (err) 
-			return FALSE;
-		}
+                if (err) 
+                    return FALSE;
+            }
 #endif
-	    } else
-		nPtr->NeoMMIOBase =
-		    xf86MapVidMem(pScrn->scrnIndex,
-				  VIDMEM_MMIO, nPtr->NeoMMIOAddr,
-				  0x200000L);
-	    if (nPtr->NeoMMIOBase == NULL)
-	        return FALSE;
-	}
-
-	if (nPtr->pEnt->location.type == BUS_PCI)
-
-#ifndef XSERVER_LIBPCIACCESS
-	    nPtr->NeoFbBase =
-		xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
-			      nPtr->PciTag,
-			      (unsigned long)nPtr->NeoLinearAddr,
-			      nPtr->NeoFbMapSize);
-#else
-	{
-	    void** result = (void**)&nPtr->NeoFbBase;
-	    int err = pci_device_map_range(nPtr->PciInfo,
-					   nPtr->NeoLinearAddr,
-					   nPtr->NeoFbMapSize,
-					   PCI_DEV_MAP_FLAG_WRITABLE |
-					   PCI_DEV_MAP_FLAG_WRITE_COMBINE,
-					   result);
-	    if (err)
-		return FALSE;
-	}
-#endif
-	else
-	    nPtr->NeoFbBase =
-		xf86MapVidMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
-			      (unsigned long)nPtr->NeoLinearAddr,
-			      nPtr->NeoFbMapSize);
-	if (nPtr->NeoFbBase == NULL)
-	    return FALSE;
-    } else {
-	/* In paged mode Base is the VGA window at 0xA0000 */
-	nPtr->NeoFbBase = hwp->Base;
+        } else
+            nPtr->NeoMMIOBase =
+                xf86MapVidMem(pScrn->scrnIndex,
+                              VIDMEM_MMIO, nPtr->NeoMMIOAddr,
+                              0x200000L);
+        if (nPtr->NeoMMIOBase == NULL)
+            return FALSE;
     }
+
+    if (nPtr->pEnt->location.type == BUS_PCI)
+
+#ifndef XSERVER_LIBPCIACCESS
+        nPtr->NeoFbBase =
+            xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
+                          nPtr->PciTag,
+                          (unsigned long)nPtr->NeoLinearAddr,
+                          nPtr->NeoFbMapSize);
+#else
+    {
+        void** result = (void**)&nPtr->NeoFbBase;
+        int err = pci_device_map_range(nPtr->PciInfo,
+                                       nPtr->NeoLinearAddr,
+                                       nPtr->NeoFbMapSize,
+                                       PCI_DEV_MAP_FLAG_WRITABLE |
+                                       PCI_DEV_MAP_FLAG_WRITE_COMBINE,
+                                       result);
+        if (err)
+            return FALSE;
+    }
+#endif
+    else
+        nPtr->NeoFbBase =
+            xf86MapVidMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
+                          (unsigned long)nPtr->NeoLinearAddr,
+                          nPtr->NeoFbMapSize);
+    if (nPtr->NeoFbBase == NULL)
+        return FALSE;
     return TRUE;
 }
 
@@ -2145,32 +1998,30 @@ neoUnmapMem(ScrnInfoPtr pScrn)
 {
     NEOPtr nPtr = NEOPTR(pScrn);
 
-    if (!nPtr->noLinear) {
 #ifndef XSERVER_LIBPCIACCESS
-      if (nPtr->NeoMMIOBase)
-	  xf86UnMapVidMem(pScrn->scrnIndex, (pointer)nPtr->NeoMMIOBase,
-			  0x200000L);
+    if (nPtr->NeoMMIOBase)
+        xf86UnMapVidMem(pScrn->scrnIndex, (pointer)nPtr->NeoMMIOBase,
+                        0x200000L);
 #else
-      if (nPtr->NeoMMIOBase)
-	  pci_device_unmap_range(nPtr->PciInfo, (pointer)nPtr->NeoMMIOBase, 0x200000L);
+    if (nPtr->NeoMMIOBase)
+        pci_device_unmap_range(nPtr->PciInfo, (pointer)nPtr->NeoMMIOBase, 0x200000L);
 #endif
-      nPtr->NeoMMIOBase = NULL;
+    nPtr->NeoMMIOBase = NULL;
 #ifndef XSERVER_LIBPCIACCESS
-      if (nPtr->NeoMMIOBase2)
-	  xf86UnMapVidMem(pScrn->scrnIndex, (pointer)nPtr->NeoMMIOBase2,
-			  0x100000L);
+    if (nPtr->NeoMMIOBase2)
+        xf86UnMapVidMem(pScrn->scrnIndex, (pointer)nPtr->NeoMMIOBase2,
+                        0x100000L);
 #else
-      if (nPtr->NeoMMIOBase2)
-	  pci_device_unmap_range(nPtr->PciInfo, (pointer)nPtr->NeoMMIOBase2, 0x100000L);
+    if (nPtr->NeoMMIOBase2)
+        pci_device_unmap_range(nPtr->PciInfo, (pointer)nPtr->NeoMMIOBase2, 0x100000L);
 #endif
-      nPtr->NeoMMIOBase2 = NULL;
+    nPtr->NeoMMIOBase2 = NULL;
 #ifndef XSERVER_LIBPCIACCESS
-      xf86UnMapVidMem(pScrn->scrnIndex, (pointer)nPtr->NeoFbBase,
-		    nPtr->NeoFbMapSize); 
+    xf86UnMapVidMem(pScrn->scrnIndex, (pointer)nPtr->NeoFbBase,
+                    nPtr->NeoFbMapSize); 
 #else
-      pci_device_unmap_range(nPtr->PciInfo, (pointer)nPtr->NeoFbBase, nPtr->NeoFbMapSize);
+    pci_device_unmap_range(nPtr->PciInfo, (pointer)nPtr->NeoFbBase, nPtr->NeoFbMapSize);
 #endif
-    }
     nPtr->NeoFbBase = NULL;
     
     return TRUE;
@@ -2566,7 +2417,7 @@ neoRestore(ScrnInfoPtr pScrn, vgaRegPtr VgaReg, NeoRegPtr restore,
      * In some rare cases a lockup might occur if we don't delay
      * here. (Reported by Miles Lane)
      */
-    xf86UDelay(200000);
+    usleep(200000);
     /*
      * Disable horizontal and vertical graphics and text expansions so
      * that vgaHWRestore works properly.
@@ -2579,7 +2430,7 @@ neoRestore(ScrnInfoPtr pScrn, vgaRegPtr VgaReg, NeoRegPtr restore,
      * Sleep for 200ms to make sure that the two operations above have
      * had time to take effect.
      */
-    xf86UDelay(200000);
+    usleep(200000);
     /*
      * This function handles restoring the generic VGA registers.  */
     vgaHWRestore(pScrn, VgaReg,
@@ -2742,7 +2593,7 @@ neoModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
      */
     NeoStd->Attribute[16] = 0x01;
 
-    switch (pScrn->depth) { /*@!@*/
+    switch (pScrn->depth) {
     case  8 :
 	NeoStd->CRTC[0x13] = pScrn->displayWidth >> 3;
 	NeoNew->ExtCRTOffset   = pScrn->displayWidth >> 11;
@@ -2784,8 +2635,7 @@ neoModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 
     /* If they are used, enable linear addressing and/or enable MMIO. */
     NeoNew->SysIfaceCntl2 = 0x00;
-    if (!nPtr->noLinear)
-	NeoNew->SysIfaceCntl2 |= 0x80;
+    NeoNew->SysIfaceCntl2 |= 0x80;
     if (!nPtr->noMMIO)
 	NeoNew->SysIfaceCntl2 |= 0x40;
 
@@ -3206,7 +3056,6 @@ neoDoDDC2(ScrnInfoPtr pScrn)
 
     VGAwGR(0x09,0x26);
     if (xf86LoadSubModule(pScrn, "i2c")) {
-        xf86LoaderReqSymLists(i2cSymbols, NULL);
 	if (neo_I2CInit(pScrn)) {
 	    ret = xf86SetDDCproperties(pScrn,xf86PrintEDID(xf86DoEDID_DDC2(
 					      pScrn->scrnIndex,nPtr->I2C)));
@@ -3227,7 +3076,6 @@ neoDoDDCVBE(ScrnInfoPtr pScrn)
 
     VGAwGR(0x09,0x26);
     if (xf86LoadSubModule(pScrn, "vbe")) {
-	xf86LoaderReqSymLists(vbeSymbols, NULL);
         if ((pVbe = VBEInit(NULL,nPtr->pEnt->index))) {
 	  ret = xf86SetDDCproperties(
 				     pScrn,xf86PrintEDID(vbeDoEDID(pVbe,NULL)));
